#include "mpr121.h"
#include <Wire.h>
#include <MIDI.h>
#include <LiquidCrystal.h>
#include <EEPROM.h>

//inicializar libreria LCD
LiquidCrystal lcd(13, 12, 11, 10, 9, 8);

//Pines encoder
#define pin_encoder_1 7
#define pin_encoder_2 6

//Pines botones           BOTON 1->NEXT    BOTON 2->PREVIOUS
#define pin_button_1 5
#define pin_button_2 4

//Pines de interrupcion de los sensores
#define pin_sensor_1 2
#define pin_sensor_2 3

// variables de ajuste de cada pin
byte CDC_sensor[20] = {0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28};

//Variables del encoder
byte encoder_position = 0;
boolean encoder_1_anterior;
boolean encoder_2_anterior;
boolean encoder_1_actual;
boolean encoder_2_actual;


//contador
int i = 0;

//variables de interrupcion
static boolean touch_1st = false;
static boolean touch_2nd = false;


//Estado de los pines de cada sensor
boolean valor_pines[20];



//variables MIDI
byte nota_midi[20] = {36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55};
byte control[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
//para el preset multitrack
byte multitrack_fader[4] = {0, 0, 0, 0};

//Variables de seleccion de programa
boolean toca_ya_o_no = false;

//Variables de ajuste
boolean ajusta_preset_o_no = false;
byte numero_preset_ajuste = 0;

//numero de preset:     11->Step sequencer  12->keyboard   13->MPC-Style  0->Multitrack
byte numero_preset = 0;



//declare reset function at address 0
void(* resetFunc) (void) = 0;


void setup()
{
  
  //Activa MIDI
  MIDI.begin(1);
  
  //Activa I2C
  Wire.begin();
  
  //Serial.begin(9600);
  
  //Activa LCD
  lcd.begin(16, 2);
  
  //Setup de los sensores
  mpr121_1st_sensor_setup();
  mpr121_2nd_sensor_setup();
  
  //Pines de entrada del encoder
  pinMode(pin_encoder_1,INPUT);
  pinMode(pin_encoder_2,INPUT);
  //Resistencias Pull-up en el encoder
  digitalWrite(pin_encoder_1, HIGH);
  digitalWrite(pin_encoder_2, HIGH);
  
  //Pines de entrada de botones
  pinMode(pin_button_1,INPUT);
  pinMode(pin_button_2,INPUT);
  
  //Pines de interrupcion de los sensores
  pinMode(pin_sensor_1, INPUT);
  pinMode(pin_sensor_2, INPUT);
  
  //Resistencias de Pull-up en los sensores
  digitalWrite(pin_sensor_1, HIGH);
  digitalWrite(pin_sensor_2, HIGH);
  
  //Definicion de las interrupciones al tocar los pines
  attachInterrupt(0, touching_1st_sensor, FALLING);
  attachInterrupt(1, touching_2nd_sensor, FALLING);
  

  
  //inicializacion de las variables del encoder
  encoder_1_actual = digitalRead(pin_encoder_1);
  encoder_2_actual = digitalRead(pin_encoder_2);
  encoder_1_anterior = encoder_1_actual;
  encoder_2_anterior = encoder_2_actual;
  
  

  
  

  //empieza el programa principal
  
  lcd.print("WELLCOME!");
  delay(3000);  
  
  //escoje preset
    lcd.clear();
    lcd.print("CHOOSE PRESET");
    while(digitalRead(pin_button_1) == false) {
    
    //Mirar siempre si hay reset
    if (digitalRead(pin_button_2) == true) {
      resetFunc();
    }
    
      //lectura de los pines del encoder
      encoder_1_actual = digitalRead(pin_encoder_1);
      encoder_2_actual = digitalRead(pin_encoder_2);
  
      //si cambia, aumenta o reduce  
      if(encoder_1_actual != encoder_1_anterior) {
        if((encoder_1_actual == encoder_2_actual) && (encoder_1_actual == 1)) {
          numero_preset ++;
        }
        else {
          if (encoder_1_actual == 1) {            
              numero_preset --;
            }
        }
        encoder_1_anterior = encoder_1_actual;
        encoder_2_anterior = encoder_2_actual;
        
        //solo van de 0 a 13
        if (numero_preset == 255) {
          numero_preset = 13;
        }
        if (numero_preset > 13) {
          numero_preset = 0;
        }
        
        //imprime el preset ecogido
        if (numero_preset == 11) {
          lcd.clear();
          lcd.print("Step Sequencer");
        }
        if (numero_preset == 12) {
          lcd.clear();
          lcd.print("keyboard");
        }
        if (numero_preset == 13) {
          lcd.clear();
          lcd.print("MPC-Style");
        }
        if (numero_preset == 0) {
          lcd.clear();
          lcd.print("Multi-Track");
        }
        if ((numero_preset > 0) && (numero_preset < 11)) {
          lcd.clear();
          lcd.print("Preset ");
          lcd.print(numero_preset);
        }
        
      }
      
    }
    
    //se presiona el boton 1 cuando se ha escogido el programa
    while(digitalRead(pin_button_1) == true) {
    }
    delay(300);
    
    //ajuste de cada preset
    if (numero_preset == 11) {
      //Step Sequencer --direccion EEPROM 60-80
      int direccion_EEPROM = 60;
      for(i=0; i<20; i++) {
        nota_midi[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      }
      //ajuste de la corriente de carga de cada pin para step-sequencer: direccion EEPROM 140-160 
      direccion_EEPROM = 140; 
      for(i=0; i<20; i++) {
        CDC_sensor[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      } 
      //Setup de los sensores
      mpr121_1st_sensor_setup();
      mpr121_2nd_sensor_setup();  
    }
    
    if (numero_preset == 12) {
      //Keyboard  --- direccion EEPROM: 40-60
      int direccion_EEPROM = 40;
      for(i=0; i<13; i++) {
        nota_midi[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      }
      //ajuste de la corriente de carga de cada pin para keyboard: direccion EEPROM 100-120 
      direccion_EEPROM = 100; 
      for(i=0; i<20; i++) {
        CDC_sensor[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      } 
      //Setup de los sensores
      mpr121_1st_sensor_setup();
      mpr121_2nd_sensor_setup();  
    }
    
        
    if (numero_preset == 13) {
      //MPC-Style --direccion EEPROM 20-40
      int direccion_EEPROM = 20;
      for(i=0; i<20; i++) {
        nota_midi[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      }
      //ajuste de la corriente de carga de cada pin para MPC-Style: direccion EEPROM 120-140 
      direccion_EEPROM = 120; 
      for(i=0; i<20; i++) {
        CDC_sensor[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      } 
      //Setup de los sensores
      mpr121_1st_sensor_setup();
      mpr121_2nd_sensor_setup();  
    }
    
    
    if (numero_preset == 0) {
      //DJ-Mixer --direccion EEPROM 0-20
      int direccion_EEPROM = 0;
      for(i=0; i<20; i++) {
        nota_midi[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      }
      //ajuste de la corriente de carga de cada pin para DJ-Mixer: direccion EEPROM 80-100 
      direccion_EEPROM = 80; 
      for(i=0; i<20; i++) {
        CDC_sensor[i] = EEPROM.read(direccion_EEPROM);
        direccion_EEPROM++;
      } 
      //Setup de los sensores
      mpr121_1st_sensor_setup();
      mpr121_2nd_sensor_setup();  
    }
    
  lcd.clear();
  lcd.print("Adjust contact?");
  delay(1000);
  while(digitalRead(pin_button_1) == false) {
    
    //Mirar siempre si hay reset
    if (digitalRead(pin_button_2) == true) {
      resetFunc();
    }
    
    //lectura de los pines del encoder
    encoder_1_actual = digitalRead(pin_encoder_1);
    encoder_2_actual = digitalRead(pin_encoder_2);
    
    //si cambia, aumenta o reduce  
    if(encoder_1_actual != encoder_1_anterior)  {
      if (encoder_1_actual == 1){
        toca_ya_o_no = !toca_ya_o_no;
      }
      encoder_1_anterior = encoder_1_actual;
      encoder_2_anterior = encoder_2_actual;
      
      //imprime programa escogido en el puerto serie
      if (toca_ya_o_no == true) {
        lcd.clear();
        lcd.print("YES");
      }
      else {
        lcd.clear();
        lcd.print("NO");
      }
    }  
  } 
  //se presiona el boton 1 para escoger el programa
  while(digitalRead(pin_button_1) == true) {
  }
  delay(300);
    
  
  

  //Si el programa escogido es NEW PROGRAM, haz todo lo siguiente
  if (toca_ya_o_no == true) {
    lcd.clear();
    lcd.print("ADJUST CONTACT");
    delay(1000);
    for(i=1; i<13; i++){
      if (i == 12)
      {
        i=0;
      }
      //empieza a ajustar el contacto
      lcd.clear();
      lcd.print("Adj: Touch ");
      if (i == 0)
      {
        lcd.print("12");
      }
      else
      {
        lcd.print(i);
      }
      delay(500);
       while(digitalRead(pin_button_1) == false) {
          //Mirar siempre si hay reset
          if (digitalRead(pin_button_2) == true) {
          resetFunc();
          }
                   
          //lee el valor de los pines
          Wire.requestFrom(0x5A,2);     
          byte LSB = Wire.read();
          byte MSB = Wire.read();          
          uint16_t touched_1st = ((MSB << 8) | LSB); 
          
          if(touched_1st & (1<<i)){
        
            if(valor_pines[i] == 0){
              //tocado
              lcd.clear();
              if (i == 0)
              {
                lcd.print("12");
              }
              else
              {
                lcd.print(i);
              }
              lcd.print(" touched");
            }      
            valor_pines[i] = 1;      
          }
          else{
            if(valor_pines[i] == 1){
              //soltado
              lcd.clear();
              if (i == 0)
              {
                lcd.print("12");
              }
              else
              {
                lcd.print(i);
              }
              lcd.print(" released");
           }       
            valor_pines[i] = 0;
          }
      //gira encoder para ajustar el touch/release del pin
      encoder_1_actual = digitalRead(pin_encoder_1);
      encoder_2_actual = digitalRead(pin_encoder_2);
  
      //si cambia, aumenta o reduce  
      if(encoder_1_actual != encoder_1_anterior) {
        if((encoder_1_actual == encoder_2_actual) && (encoder_1_actual == 1)) {
          CDC_sensor[i] ++;
          mpr121_1st_sensor_setup();
          valor_pines[i] = 0;
          lcd.clear();
          lcd.print(CDC_sensor[i]);
          delay(300);
          lcd.clear();
          lcd.print("Adj: Touch ");
          if (i == 0)
          {
            lcd.print("12");
          }
          else
          {
            lcd.print(i);
          }
          delay(300);
          //Serial.print(CDC_sensor[i], HEX);
 
        }
        else {
          if (encoder_1_actual == 1) { 
             if(CDC_sensor[i] != 0x3F) {           
                CDC_sensor[i] --;
                mpr121_1st_sensor_setup();
                valor_pines[i] = 0;
                lcd.clear();
                lcd.print(CDC_sensor[i]);
                delay(300);
                lcd.clear();
                lcd.print("Adj: Touch ");
                if (i == 0)
                {
                  lcd.print("12");
                }
                else
                {
                  lcd.print(i);
                }
                delay(300);
                //Serial.print(CDC_sensor[i], HEX);
             }
 
            }
        }
        encoder_1_anterior = encoder_1_actual;
        encoder_2_anterior = encoder_2_actual;
     }
     
  }
  while(digitalRead(pin_button_1) == true) {
  }
  delay(300);
  
  if (i == 0)
      {
        i=12;
      }  
 }
    //ajusta ahora los pines del 2do sensor
    for(i=0; i<8; i++){
      //empieza a ajustar el contacto
      lcd.clear();
      lcd.print("Adj: Touch ");
      lcd.print(i+13);
      delay(500);
       while(digitalRead(pin_button_1) == false) {
          //Mirar siempre si hay reset
          if (digitalRead(pin_button_2) == true) {
          resetFunc();
          }
                   
          //lee el valor de los pines
          Wire.requestFrom(0x5B,1);              
          uint8_t touched_2nd = Wire.read(); 
          
          if(touched_2nd & (1<<i)){
        
            if(valor_pines[i+12] == 0){
              //tocado
              lcd.clear();
              lcd.print(i+13);
              lcd.print(" touched");
            }      
            valor_pines[i+12] = 1;      
          }
          else{
            if(valor_pines[i+12] == 1){
              //soltado
              lcd.clear();
              lcd.print(i+13);
              lcd.print(" released");
           }       
            valor_pines[i+12] = 0;
          }
      //gira encoder para ajustar el touch/release del pin
      encoder_1_actual = digitalRead(pin_encoder_1);
      encoder_2_actual = digitalRead(pin_encoder_2);
  
      //si cambia, aumenta o reduce  
      if(encoder_1_actual != encoder_1_anterior) {
        if((encoder_1_actual == encoder_2_actual) && (encoder_1_actual == 1)) {
          CDC_sensor[i+12] ++;
          mpr121_2nd_sensor_setup();
          valor_pines[i+12] = 0;
          lcd.clear();
          lcd.print(CDC_sensor[i+12]);
          delay(300);
          lcd.clear();
          lcd.print("Adj: Touch ");
          lcd.print(i+13);
          delay(300);
          //Serial.print(CDC_sensor[i+12], HEX);
        }
        else {
          if (encoder_1_actual == 1) {
              if(CDC_sensor[i+12] != 0x3F) {            
                CDC_sensor[i+12] --;
                mpr121_2nd_sensor_setup();
                valor_pines[i+12] = 0;
                lcd.clear();
                lcd.print(CDC_sensor[i+12]);
                delay(300);
                lcd.clear();
                lcd.print("Adj: Touch ");
                lcd.print(i+13);
                delay(300);
                //Serial.print(CDC_sensor[i+12], HEX);
              }
            }
        }
        encoder_1_anterior = encoder_1_actual;
        encoder_2_anterior = encoder_2_actual;
     }
     
  }
  while(digitalRead(pin_button_1) == true) {
  }
  delay(300);
    
 }
 
 //pregunta si quieren guardarse los nuevos ajustes 
 lcd.clear();
 lcd.print("save to preset?");
 delay(1000);
 
  while(digitalRead(pin_button_1) == false) {
    
    //Mirar siempre si hay reset
    if (digitalRead(pin_button_2) == true) {
      resetFunc();
    }
    
    //lectura de los pines del encoder
    encoder_1_actual = digitalRead(pin_encoder_1);
    encoder_2_actual = digitalRead(pin_encoder_2);
    
    //si cambia, aumenta o reduce  
    if(encoder_1_actual != encoder_1_anterior)  {
      if (encoder_1_actual == 1){
        ajusta_preset_o_no = !ajusta_preset_o_no;
      }
      encoder_1_anterior = encoder_1_actual;
      encoder_2_anterior = encoder_2_actual;
      
      //imprime programa escogido en el puerto serie
      if (ajusta_preset_o_no == true) {
        lcd.clear();
        lcd.print("YES");
      }
      else {
        lcd.clear();
        lcd.print("NO");
      }
    }  
  } 
  //se presiona el boton 1 para escoger el programa
  while(digitalRead(pin_button_1) == true) {
  }
  delay(300);
  
  //Si el programa escogido es presets, haz todo lo siguiente
  if (ajusta_preset_o_no == true) {
    lcd.clear();
    lcd.print("CHOOSE SAVE PRESET");
    while(digitalRead(pin_button_1) == false) {
    
    //Mirar siempre si hay reset
    if (digitalRead(pin_button_2) == true) {
      resetFunc();
    }
    
      //lectura de los pines del encoder
      encoder_1_actual = digitalRead(pin_encoder_1);
      encoder_2_actual = digitalRead(pin_encoder_2);
  
      //si cambia, aumenta o reduce  
      if(encoder_1_actual != encoder_1_anterior) {
        if((encoder_1_actual == encoder_2_actual) && (encoder_1_actual == 1)) {
          numero_preset_ajuste ++;
        }
        else {
          if (encoder_1_actual == 1) {            
              numero_preset_ajuste --;
            }
        }
        encoder_1_anterior = encoder_1_actual;
        encoder_2_anterior = encoder_2_actual;
        
        //solo van de 0 a 13
        if (numero_preset_ajuste == 255) {
          numero_preset_ajuste = 13;
        }
        if (numero_preset_ajuste > 13) {
          numero_preset_ajuste = 0;
        }
        
        //imprime el preset ecogido
        if (numero_preset_ajuste == 11) {
          lcd.clear();
          lcd.print("Step Sequencer");
        }
        if (numero_preset_ajuste == 12) {
          lcd.clear();
          lcd.print("keyboard");
        }
        if (numero_preset_ajuste == 13) {
          lcd.clear();
          lcd.print("MPC-Style");
        }
        if (numero_preset_ajuste == 0) {
          lcd.clear();
          lcd.print("Multi-Track");
        }
        if ((numero_preset_ajuste > 0) && (numero_preset_ajuste < 11)) {
          lcd.clear();
          lcd.print("Preset ");
          lcd.print(numero_preset_ajuste);
        }
        
      }
      
    }
    
    //se presiona el boton 1 cuando se ha escogido el programa
    while(digitalRead(pin_button_1) == true) {
    }
    delay(300);
    
    //
    if (numero_preset_ajuste == 11) {
      //Step Sequencer --  valores de CDC_sensor guardados en direccion EEPROM 140-160
      int direccion_EEPROM = 140;
      for(i=0; i<20; i++) {
        EEPROM.write(direccion_EEPROM, CDC_sensor[i]);
        direccion_EEPROM++;
      }      
    }
    
    if (numero_preset_ajuste == 12) {
      //Keyboard  --- valores de CDC_sensor guardados en direccion EEPROM: 100-120
      int direccion_EEPROM = 100;
      for(i=0; i<20; i++) {
        EEPROM.write(direccion_EEPROM, CDC_sensor[i]);
        direccion_EEPROM++;
      }
    }
        
    if (numero_preset_ajuste == 13) {
      //MPC-Style -- valores de CDC_sensor guardados en direccion EEPROM 120-140
      int direccion_EEPROM = 120;
      for(i=0; i<20; i++) {
        EEPROM.write(direccion_EEPROM, CDC_sensor[i]);
        direccion_EEPROM++;
      }
    }
    
    if (numero_preset_ajuste == 0) {
      //Multi-Track -- valores de CDC_sensor guardados en direccion EEPROM 80-100
      int direccion_EEPROM = 80;
      for(i=0; i<20; i++) {
        EEPROM.write(direccion_EEPROM, CDC_sensor[i]);
        direccion_EEPROM++;
      }
    }
    else
    {
      //Preset 1 to 10 -- valodres de CDC_sensor guardados en direccion EEPROM 140+(numero_preset_ajuste*20)
      int direccion_EEPROM = 140+(numero_preset_ajuste*20);
      for(i=0; i<20; i++)
      {
        EEPROM.write(direccion_EEPROM, CDC_sensor[i]);
        direccion_EEPROM++;
      }
    }
    lcd.clear();
    lcd.print("SAVED!");
    delay(1000);
  }
 //----------------------------------------------------------------------------------------
 
  }
    
    
    lcd.clear();
    lcd.print("READY!");
    delay(2000);
    if (numero_preset == 11) {
      lcd.clear();
      lcd.print("Step Sequencer");
    }
    if (numero_preset == 12) {
      lcd.clear();
      lcd.print("keyboard");
    }
    if (numero_preset == 13) {
      lcd.clear();
      lcd.print("MPC-Style");
    }
    if (numero_preset == 0) {
      lcd.clear();
      lcd.print("Multi-Track");
    }
    if ((numero_preset > 0) && (numero_preset < 11)) {
      lcd.clear();
      lcd.print("Preset ");
      lcd.print(numero_preset);
    }


  
      
}     
      
      
     


void loop()
{
  //preguntar si se tocan pines del sonsor 1
  if (touch_1st == true) {
    
    Wire.requestFrom(0x5A,2); 
    
    byte LSB = Wire.read();
    byte MSB = Wire.read();
    
    uint16_t touched_1st = ((MSB << 8) | LSB); 

    //mirar si el preset es el multi-track, que no son todo notas midi
    switch (numero_preset)
    {
      case 0: //en caso de que sea multitrack
       // Comprobar que pines se han tocado
        for (int i=0; i < 12; i++){  
          if(touched_1st & (1<<i)){
            
            if(valor_pines[i] == 0){
              switch (i) 
              {
                case 1:
                  if (multitrack_fader[0] > 10)
                  {
                    multitrack_fader[0] = multitrack_fader[0]-10;
                  }
                  else
                  {
                    multitrack_fader[0] = 0;
                  }
                  MIDI.sendControlChange(1,multitrack_fader[0],1);
                  break;
                case 2:
                if (multitrack_fader[0] < 127)
                  {
                    multitrack_fader[0] = multitrack_fader[0]+10;
                  }
                  if (multitrack_fader[0] > 127)
                  {
                    multitrack_fader[0] = 127;
                  }
                  MIDI.sendControlChange(1,multitrack_fader[0],1);
                  break;
                case 9:
                if (multitrack_fader[1] < 127)
                  {
                    multitrack_fader[1] = multitrack_fader[1]+10;
                  }
                  if (multitrack_fader[1] > 127)
                  {
                    multitrack_fader[1] = 127;
                  }
                  MIDI.sendControlChange(2,multitrack_fader[1],1);
                  break;
                case 10:
                if (multitrack_fader[1] > 10)
                  {
                    multitrack_fader[1] = multitrack_fader[1]-10;
                  }
                  else
                  {
                    multitrack_fader[1] = 0;
                  }
                  MIDI.sendControlChange(2,multitrack_fader[1],1);
                  break;
                default:
                  MIDI.sendNoteOn(nota_midi[i],100,1);
              }
            }
            valor_pines[i] = 1;      
          }
          else{
            if(valor_pines[i] == 1){
              if ((i == 0)||(i == 3)||(i == 4)||(i == 5)||(i == 6)||(i == 7)||(i == 8)||(i == 11))
              {
                //envia midi
                MIDI.sendNoteOff(nota_midi[i],0,1);
              }
           }       
            valor_pines[i] = 0;
          }
        
        }  
        break;
      default: //cualquier oto preset
        // Comprobar que pines se han tocado
        for (int i=0; i < 12; i++){  
          if(touched_1st & (1<<i)){
            
            if(valor_pines[i] == 0){
              //envia midi
              MIDI.sendNoteOn(nota_midi[i],100,1);
            }      
            valor_pines[i] = 1;      
          }
          else{
            if(valor_pines[i] == 1){
              //envia midi
              MIDI.sendNoteOff(nota_midi[i],0,1);
           }       
            valor_pines[i] = 0;
          }
        
        }
    }
    touch_1st = false;
    
  }
  else {
    //preguntar si se tocan pines del sonsor 2
    if (touch_2nd == true) {
    
      Wire.requestFrom(0x5B,1); 
    
      uint8_t touched_2nd = Wire.read(); 
      
      //El mapeo MIDI de los pines del 2do sensor cambia en funcion del preset escogido
      
      switch (numero_preset) {
      //multi-track
      case 0:
        for (int i=0; i < 8; i++){  
          if(touched_2nd & (1<<i)){
        
            if(valor_pines[i+12] == 0){
              switch (i) 
              {
                case 1:
                if (multitrack_fader[2] < 127)
                  {
                    multitrack_fader[2] = multitrack_fader[2]+10;
                  }
                  if (multitrack_fader[2] > 127)
                  {
                    multitrack_fader[2] = 127;
                  }
                  MIDI.sendControlChange(3,multitrack_fader[2],1);
                  break;
                case 2:
                if (multitrack_fader[2] > 10)
                  {
                    multitrack_fader[2] = multitrack_fader[2]-10;
                  }
                  else
                  {
                    multitrack_fader[2] = 0;
                  }
                  MIDI.sendControlChange(3,multitrack_fader[2],1);
                  break;
                case 6:
                if (multitrack_fader[3] < 127)
                  {
                    multitrack_fader[3] = multitrack_fader[3]+10;
                  }
                  if (multitrack_fader[3] > 127)
                  {
                    multitrack_fader[3] = 127;
                  }
                  MIDI.sendControlChange(4,multitrack_fader[3],1);
                  break;
                case 7:
                if (multitrack_fader[3] > 10)
                  {
                    multitrack_fader[3] = multitrack_fader[3]-10;
                  }
                  else
                  {
                    multitrack_fader[3] = 0;
                  }
                  MIDI.sendControlChange(4,multitrack_fader[3],1);
                  break;
                default:
                  MIDI.sendNoteOn(nota_midi[i+12],100,1);
              }
            }  
            valor_pines[i+12] = 1;      
          }
          else{
            if(valor_pines[i+12] == 1){
              if ((i==0)||(i==3)||(i==4)||(i==5))
              {
                //envia midi
                MIDI.sendNoteOff(nota_midi[i+12],0,1);
              }
           }       
            valor_pines[i+12] = 0;
          }
        }
        break;
        
      //STEP SEQUENCER
      case 11:
        for (int i=0; i < 8; i++){  
          if(touched_2nd & (1<<i)){
        
            if(valor_pines[i+12] == 0){
              //envia midi
              MIDI.sendNoteOn(nota_midi[i+12],100,1);
            }      
            valor_pines[i+12] = 1;      
          }
          else{
            if(valor_pines[i+12] == 1){
              //envia midi
              MIDI.sendNoteOff(nota_midi[i+12],0,1);
           }       
            valor_pines[i+12] = 0;
          }
        }
      break;
      
      //KEYBOARD
      case 12:
        for (int i=0; i < 8; i++){  
          if(touched_2nd & (1<<i)){
            if(valor_pines[i+12] == 0)
            {
              //pin 12(12+0) es la nota midi C alta
              if (i == 0) {
                MIDI.sendNoteOn(nota_midi[12],100,1);
              }
              //pines 19(12+7) y 18(12+6) son los de OCTAVA!!! 
              if (i == 7) {
                for(i=0; i<13; i++){
                  nota_midi[i] = nota_midi[i]+12;
                }
              }
              if (i == 6) {
                for(i=0; i<13; i++){
                  nota_midi[i] = nota_midi[i]-12;
                }
              }
              //pines 17(12+4) y 16(12+4) son los de PITCHBEND!!!
              if (i == 5){
                MIDI.sendPitchBend(1366,1);
              }
              if (i == 4){
                MIDI.sendPitchBend(-1366,1);
              }
            }      
            valor_pines[i+12] = 1;      
          }
          else{
            if(valor_pines[i+12] == 1){
              //pin 12(12+0) es la nota midi C alta
              if (i == 0) {
                MIDI.sendNoteOff(nota_midi[12],0,1);
              }
              //pines 19(12+7) y 18(12+6) son los de PITCHBEND!!! no estoy seguro de que sean estos
              if ((i == 5) || (i == 4)){
                MIDI.sendPitchBend(0,1);
              }
            }          
          valor_pines[i+12] = 0;
          }
      
        }
      break;
      
      //MPC-STYLE    
      case 13:
        for (int i=0; i < 8; i++){  
          if(touched_2nd & (1<<i)){
        
            if(valor_pines[i+12] == 0){
              //envia midi
              MIDI.sendNoteOn(nota_midi[i+12],100,1);
            }      
            valor_pines[i+12] = 1;      
          }
          else{
            if(valor_pines[i+12] == 1){
              //envia midi
              MIDI.sendNoteOff(nota_midi[i+12],0,1);
           }       
            valor_pines[i+12] = 0;
          }
        }
      break;
      
      //cualquier otro
      default:
        for (int i=0; i < 8; i++){  
          if(touched_2nd & (1<<i)){
        
            if(valor_pines[i+12] == 0){
              //envia midi
              MIDI.sendNoteOn(nota_midi[i+12],100,1);
            }      
            valor_pines[i+12] = 1;      
          }
          else{
            if(valor_pines[i+12] == 1){
              //envia midi
              MIDI.sendNoteOff(nota_midi[i+12],0,1);
           }       
            valor_pines[i+12] = 0;
          }
        }
      
      }
        
      touch_2nd = false;
      
    }
    else {
      reset_y_encoder();
    }
  }
}

void reset_y_encoder() {

  //Mirar siempre si hay reset
  if (digitalRead(pin_button_2) == true) {
    resetFunc();
  }
  if ((touch_1st == false) && (touch_2nd == false)) {
    //lectura de los pines del encoder
    encoder_1_actual = digitalRead(pin_encoder_1);
    if (encoder_1_actual != encoder_1_anterior) {                         
      //si cambia, aumenta o reduce  
      if((touch_1st == false) && (touch_2nd == false)) {
        encoder_2_actual = digitalRead(pin_encoder_2);
        if ((touch_1st == false) && (touch_2nd == false)) {         
          if((encoder_1_actual == encoder_2_actual) && (encoder_1_actual == 1)) {
            encoder_position ++;
          }
          else {
            if (encoder_1_actual == 1) {            
              encoder_position --;
            }
          }
          encoder_1_anterior = encoder_1_actual;
          encoder_2_anterior = encoder_2_actual;
          
          if ((touch_1st == false) && (touch_2nd == false)) {
            if(encoder_1_actual == 1){    
              //envia midi
              MIDI.sendControlChange(10,encoder_position,1);
              
            }
          }
        }
      }
    }
  }

}




void touching_1st_sensor()
  {
    touch_1st = true;
  }
  
  void touching_2nd_sensor()
  {
    touch_2nd = true;
  }

  
  
//Funcion necesaria para el setup de los sensores
void set_register(int address, unsigned char r, unsigned char v){
    Wire.beginTransmission(address);
    Wire.write(r);
    Wire.write(v);
    Wire.endTransmission();
}


  
//Setup del primer sensor  
void mpr121_1st_sensor_setup(void){

  set_register(0x5A, ELE_CFG, 0x00); 

  // Section A - Controls filtering when data is > baseline.
  set_register(0x5A, MHD_R, 0x01);
  set_register(0x5A, NHD_R, 0x01);
  set_register(0x5A, NCL_R, 0x00);
  set_register(0x5A, FDL_R, 0x00);

  // Section B - Controls filtering when data is < baseline.
  set_register(0x5A, MHD_F, 0x01);
  set_register(0x5A, NHD_F, 0x01);
  set_register(0x5A, NCL_F, 0xFF);
  set_register(0x5A, FDL_F, 0x02);

  // Section C - Sets touch and release thresholds for each electrode
  set_register(0x5A, ELE0_T, TOU_THRESH);
  set_register(0x5A, ELE0_R, REL_THRESH);

  set_register(0x5A, ELE1_T, TOU_THRESH);
  set_register(0x5A, ELE1_R, REL_THRESH);

  set_register(0x5A, ELE2_T, TOU_THRESH);
  set_register(0x5A, ELE2_R, REL_THRESH);

  set_register(0x5A, ELE3_T, TOU_THRESH);
  set_register(0x5A, ELE3_R, REL_THRESH);

  set_register(0x5A, ELE4_T, TOU_THRESH);
  set_register(0x5A, ELE4_R, REL_THRESH);

  set_register(0x5A, ELE5_T, TOU_THRESH);
  set_register(0x5A, ELE5_R, REL_THRESH);

  set_register(0x5A, ELE6_T, TOU_THRESH);
  set_register(0x5A, ELE6_R, REL_THRESH);

  set_register(0x5A, ELE7_T, TOU_THRESH);
  set_register(0x5A, ELE7_R, REL_THRESH);

  set_register(0x5A, ELE8_T, TOU_THRESH);
  set_register(0x5A, ELE8_R, REL_THRESH);

  set_register(0x5A, ELE9_T, TOU_THRESH);
  set_register(0x5A, ELE9_R, REL_THRESH);

  set_register(0x5A, ELE10_T, TOU_THRESH);
  set_register(0x5A, ELE10_R, REL_THRESH);

  set_register(0x5A, ELE11_T, TOU_THRESH);
  set_register(0x5A, ELE11_R, REL_THRESH);

  // Section D
  // Set the Filter Configuration
  // Set ESI2
  set_register(0x5A, FIL_CFG, 0x44);  //segundo filtro
  set_register(0x5A, 0x5C, 0x28);    // primer filtro (mio)
      
      //ajuste de la corrente de carga de cada pin
      set_register(0x5A, 0x5F, CDC_sensor[0]);   //el 0
      set_register(0x5A, 0x60, CDC_sensor[1]);   //el 1
      set_register(0x5A, 0x61, CDC_sensor[2]);   //el 2
      set_register(0x5A, 0x62, CDC_sensor[3]);   //el 3
      set_register(0x5A, 0x63, CDC_sensor[4]);   //el 4
      set_register(0x5A, 0x64, CDC_sensor[5]);   //el 5
      set_register(0x5A, 0x65, CDC_sensor[6]);   //el 6
      set_register(0x5A, 0x66, CDC_sensor[7]);   //el 7
      set_register(0x5A, 0x67, CDC_sensor[8]);   //el 8
      set_register(0x5A, 0x68, CDC_sensor[9]);   //el 9
      set_register(0x5A, 0x69, CDC_sensor[10]);   //el 10
      set_register(0x5A, 0x6A, CDC_sensor[11]);   //el 11

  // Section E
  // Electrode Configuration
  // Set ELE_CFG to 0x00 to return to standby mode
  set_register(0x5A, ELE_CFG, 0x0C);  // Enables all 12 Electrodes


  // Section F
  // Enable Auto Config and auto Reconfig
  /*set_register(0x5A, ATO_CFG0, 0x0B);
  set_register(0x5A, ATO_CFGU, 0xC9);  // USL = (Vdd-0.7)/vdd*256 = 0xC9 @3.3V   set_register(0x5A, ATO_CFGL, 0x82);  // LSL = 0.65*USL = 0x82 @3.3V
  set_register(0x5A, ATO_CFGT, 0xB5);*/  // Target = 0.9*USL = 0xB5 @3.3V

  set_register(0x5A, ELE_CFG, 0x0C);

}



//Setup del segundo sensor  
void mpr121_2nd_sensor_setup(void){

  set_register(0x5B, ELE_CFG, 0x00); 

  // Section A - Controls filtering when data is > baseline.
  set_register(0x5B, MHD_R, 0x01);
  set_register(0x5B, NHD_R, 0x01);
  set_register(0x5B, NCL_R, 0x00);
  set_register(0x5B, FDL_R, 0x00);

  // Section B - Controls filtering when data is < baseline.
  set_register(0x5B, MHD_F, 0x01);
  set_register(0x5B, NHD_F, 0x01);
  set_register(0x5B, NCL_F, 0xFF);
  set_register(0x5B, FDL_F, 0x02);

  // Section C - Sets touch and release thresholds for each electrode
  set_register(0x5B, ELE0_T, TOU_THRESH);
  set_register(0x5B, ELE0_R, REL_THRESH);

  set_register(0x5B, ELE1_T, TOU_THRESH);
  set_register(0x5B, ELE1_R, REL_THRESH);

  set_register(0x5B, ELE2_T, TOU_THRESH);
  set_register(0x5B, ELE2_R, REL_THRESH);

  set_register(0x5B, ELE3_T, TOU_THRESH);
  set_register(0x5B, ELE3_R, REL_THRESH);

  set_register(0x5B, ELE4_T, TOU_THRESH);
  set_register(0x5B, ELE4_R, REL_THRESH);

  set_register(0x5B, ELE5_T, TOU_THRESH);
  set_register(0x5B, ELE5_R, REL_THRESH);

  set_register(0x5B, ELE6_T, TOU_THRESH);
  set_register(0x5B, ELE6_R, REL_THRESH);

  set_register(0x5B, ELE7_T, TOU_THRESH);
  set_register(0x5B, ELE7_R, REL_THRESH);


  // Section D
  // Set the Filter Configuration
  // Set ESI2
  set_register(0x5B, FIL_CFG, 0x44);  //segundo filtro
  set_register(0x5B, 0x5C, 0x28);    // primer filtro (mio)
  
      //ajuste de la corrente de carga de cada pin
      set_register(0x5B, 0x5F, CDC_sensor[12]);   //el 12
      set_register(0x5B, 0x60, CDC_sensor[13]);   //el 13
      set_register(0x5B, 0x61, CDC_sensor[14]);   //el 14
      set_register(0x5B, 0x62, CDC_sensor[15]);   //el 15
      set_register(0x5B, 0x63, CDC_sensor[16]);   //el 16
      set_register(0x5B, 0x64, CDC_sensor[17]);   //el 17
      set_register(0x5B, 0x65, CDC_sensor[18]);   //el 18
      set_register(0x5B, 0x66, CDC_sensor[19]);   //el 19

  // Section E
  // Electrode Configuration
  // Set ELE_CFG to 0x00 to return to standby mode
  set_register(0x5B, ELE_CFG, 0x0C);  // Enables all 12 Electrodes


  // Section F
  // Enable Auto Config and auto Reconfig
  //set_register(0x5B, ATO_CFG0, 0x0B);
  //set_register(0x5B, ATO_CFGU, 0xC9);  // USL = (Vdd-0.7)/vdd*256 = 0xC9 @3.3V   //set_register(0x5B, ATO_CFGL, 0x82);  // LSL = 0.65*USL = 0x82 @3.3V
  //set_register(0x5B, ATO_CFGT, 0xB5);  // Target = 0.9*USL = 0xB5 @3.3V

  set_register(0x5B, ELE_CFG, 0x0C);

}

